function D_gen(FSP, depth, T, alpha, nu, beta, d, Gamma, gamma, M)

% Generation of single floe DTMs for floe dimensions listed in the floe
% size pool (FSP).

addpath('Routines DTM')

%% Dispersion relations

% Vertical modes for eigenfunction matching
M = [400 M];

% Open water wavenumber
k = dispersion_free_surface(alpha,M(1),depth);

% Ice/fluid surface waves 
kappa = zeros(size(FSP,1),M(1) + 3);
for p = 1:size(FSP,1)
    kappa(p,:) = dispersion_RP_VEmodel(alpha, beta(p), gamma, d(p), ...
        M(1) + 2, depth - d(p));
end

%% DTMs

% Diffraction transfer matrix of each unique floe
for i = 1:size(FSP,1)
    if exist(['Data DTM/DTMext_a' num2str(FSP(i,1)) '_h' num2str(FSP(i,2)) ...
            '_Gamma' num2str(Gamma) '_T' num2str(T) '.mat'],'file') == 0

        % Compute diffraction transfer matrix of floe 
        [DTM_ext, ~, ~, N] = Circular_Plate_Solver(depth, FSP(i,1), nu, ...
            d(i), alpha, beta(i), k, kappa(i,:), M);

        save(['Data DTM/DTMext_a' num2str(FSP(i,1)) '_h' num2str(FSP(i,2)) ...
            '_Gamma' num2str(Gamma) '_T' num2str(T) '.mat'],'DTM_ext','N')
    end

end

