function [Amps_scat_ext,N] = Multi_scat_solve(pos, FSP, R_miz, depth, T0, ...
    th0, alpha, Gamma, M)

% Amps_scat_ext = Multi_scat_solve(pos, FSP, R_miz, depth, T, ...
%     th0, alpha, Gamma, M)
%
% Solves the multiple scattering problem of a plane wave by an array of
% floating ice floes.

addpath('Routines SlabScat','FMM-routines',...
    'FMM-routines/tmatrom/TMATROM_OBJECT_ORIENTED_CORE/')

% Number of floes
Np = size(pos,1);

% Polar coordinates of centre of floe l into local coordinates 
% associated with floe j
R   = zeros(Np,Np);
psi = zeros(Np,Np);
for j = 1:Np
    for l = 1:Np
        [R(j,l),psi(j,l)] = coordinate_change(...
            [pos(j,1)*cos(pos(j,2)) pos(j,1)*sin(pos(j,2))], ...
            [pos(l,1)*cos(pos(l,2)) pos(l,1)*sin(pos(l,2))]);        
    end
end

%% Wavenumber
k = dispersion_free_surface(alpha,M,depth);

%% Single floe solutions
DTM_floes = cell(size(FSP,1),1);
NN = zeros(size(FSP,1),1);
for jj = 1:size(FSP,1)
    load(['Data DTM/DTMext_a' num2str(FSP(jj,1)) '_h' num2str(FSP(jj,2)) ...
        '_Gamma' num2str(Gamma) '_T' num2str(T0) '.mat'],'DTM_ext')
    DTM_floes{jj} = DTM_ext;
    NN(jj) = length(DTM_ext)-1;
end

% Augment pos array with number of angular modes
N = zeros(Np,1);
for ii = 1:Np
    N(ii) = NN(pos(ii,3));
end

% Concatenate all the floe DTMs into diagonal mattrix
DTMs = zeros(sum(2*N+1)*(M+1),1);
for ii = 1:Np
    for nn = -N(ii):N(ii)
        DTMs(sum(2*N(1:ii-1)+1)*(M+1)+(nn+N(ii))*(M+1)+1:...
            sum(2*N(1:ii-1)+1)*(M+1)+(nn+N(ii)+1)*(M+1)) = ...
            DTM_floes{pos(ii,3)}{abs(nn)+1};
    end
end
S = spdiags(DTMs,0,sum(2*N+1)*(M+1),sum(2*N+1)*(M+1));

%% Multiple scattering
fmm_on = 1;
tic
if fmm_on == 0
    % Interaction matrix (Graf)
    T = Inter_matrix_def(R, psi, k, M, N, Np);

    % Mat = eye(sum(2*N+1)*(M+1)) - S*T;
    % cond(Mat)
else
    % position (for FMM)
    pos_fmm = pos(:,1).*cos(pos(:,2)) + 1i*pos(:,1).*sin(pos(:,2));
    
    % Min floe diameter
    d = min(FSP(:,1));

    % FMM
    N_L = 5;%round(log2(2*pi*R_miz/d)); % Number of FMM generations/levels
    fmm = hawkins_fmm(N_L,pos_fmm,k);

    fmm.trim
    fmm.minLevel = 3;
end

% Forcing field (no evanescent modes, for now!)
f = zeros(sum(2*N+1)*(M+1),1);
for ii = 1:Np
    f(sum(2*N(1:ii-1)+1)*(M+1)+1:sum(2*N(1:ii)+1)*(M+1)) = ...
        exp(1i*k*pos(ii,1)*cos(th0-pos(ii,2)))*...
        1i.^((-N(ii):N(ii)).').*exp(-1i*(-N(ii):N(ii)).'*th0);
end
f = S*f;


% Solving for local scattered field amplitudes
gmres_on = 1;
if gmres_on == 1
    Amps_scat_ext = gmres(@mat_vec,f,min([100 length(f)]),1e-5);
else
    Mat = eye(length(f)) - S*T;
    Amps_scat_ext=Mat\f;
end
toc
% pause

    function y = mat_vec(x)
        fprintf('.');
        if fmm_on == 1
%             tic
            fmm.importExpansion(x,N)
%             toc 
%             tic
            fmm.upwardPass
%             toc
%             tic
            fmm.downwardPass
%             toc
%             tic
            z = fmm.exportExpansion(0);
%             toc
            y = x - S*z;
        else
            y = x - S*(T*x);
        end
        fprintf('.');
    end
end