function T = Inter_matrix_def(R, psi, k, M, N, Np)

% T = Inter_matrix_def(R, psi k, M, N, Np)
%
% Computes the Graf interaction matrix T for a system of Np floating ice 
% floes. The matrix T will appear in the system 
%
% (I-S*T)*b = f,
%
% where vector b contain the local scattered wave amplitudes.
% 
% Here N is a vector containing the angular modes truncation parameters for
% the different floes.


% Interaction matrix
T = zeros(sum(2*N+1)*(M+1));

%% Computation using symmetries

for j = 1:Np
    for p = j+1:Np
        % Test which angular truncation parameter is larger
        if N(j)<N(p)
            for n = -N(j):N(j)
                for s = n+1:N(p)
                    % floe j wrt floe p for mode (n,s)
                    T(sum(2*N(1:j-1)+1)*(M+1) + (n+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (n+N(j)+1)*(M+1),...
                        sum(2*N(1:p-1)+1)*(M+1) + (s+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (s+N(p)+1)*(M+1)) = ...
                        sep_matrix(R(p,j), psi(p,j), n, s, k);

                    % floe j wrt floe p for mode (-n,-s) using symmetry
                    T(sum(2*N(1:j-1)+1)*(M+1) + (-n+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (-n+N(j)+1)*(M+1),...
                        sum(2*N(1:p-1)+1)*(M+1) + (-s+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (-s+N(p)+1)*(M+1)) = ...
                        T(sum(2*N(1:j-1)+1)*(M+1) + (n+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (n+N(j)+1)*(M+1),...
                        sum(2*N(1:p-1)+1)*(M+1) + (s+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (s+N(p)+1)*(M+1))*...
                        exp(-2*1i*(s-n)*psi(p,j));

                    % floe p wrt floe j for mode (n,s)
                    T(sum(2*N(1:p-1)+1)*(M+1) + (n+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (n+N(p)+1)*(M+1),...
                        sum(2*N(1:j-1)+1)*(M+1) + (s+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (s+N(j)+1)*(M+1)) = ...
                        sep_matrix(R(j,p), psi(j,p), n, s, k);

                    % floe p wrt floe j for mode (-n,-s) using symmetry
                    T(sum(2*N(1:p-1)+1)*(M+1) + (-n+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (-n+N(p)+1)*(M+1),...
                        sum(2*N(1:j-1)+1)*(M+1) + (-s+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (-s+N(j)+1)*(M+1)) = ...
                        T(sum(2*N(1:p-1)+1)*(M+1) + (n+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (n+N(p)+1)*(M+1),...
                        sum(2*N(1:j-1)+1)*(M+1) + (s+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (s+N(j)+1)*(M+1))*...
                        exp(-2*1i*(s-n)*psi(j,p));
                end
            end
            for n = -N(j):-1
                % floe j wrt floe p for mode (n,n)
                T(sum(2*N(1:j-1)+1)*(M+1) + (n+N)*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (n+N+1)*(M+1),...
                    sum(2*N(1:p-1)+1)*(M+1) + (n+N)*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (n+N+1)*(M+1)) = ...
                    sep_matrix(R(p,j), psi(p,j), n, n, k);

                % floe j wrt floe p for mode (-n,-n) using symmetry
                T(sum(2*N(1:j-1)+1)*(M+1) + (-n+N)*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (-n+N+1)*(M+1),...
                    sum(2*N(1:p-1)+1)*(M+1) + (-n+N)*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (-n+N+1)*(M+1)) = ...
                    T(sum(2*N(1:j-1)+1)*(M+1) + (n+N)*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (n+N+1)*(M+1),...
                    sum(2*N(1:p-1)+1)*(M+1) + (n+N)*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (n+N+1)*(M+1));

                % floe p wrt floe j for mode (n,n)
                T(sum(2*N(1:p-1)+1)*(M+1) + (n+N)*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (n+N+1)*(M+1),...
                    sum(2*N(1:j-1)+1)*(M+1) + (n+N)*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (n+N+1)*(M+1)) = ...
                    sep_matrix(R(j,p), psi(j,p), n, n, k);

                % floe p wrt floe j for mode (-n,-n) using symmetry
                T(sum(2*N(1:p-1)+1)*(M+1) + (-n+N)*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (-n+N+1)*(M+1),...
                    sum(2*N(1:j-1)+1)*(M+1) + (-n+N)*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (-n+N+1)*(M+1)) = ...
                    T(sum(2*N(1:p-1)+1)*(M+1) + (n+N)*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (n+N+1)*(M+1),...
                    sum(2*N(1:j-1)+1)*(M+1) + (n+N)*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (n+N+1)*(M+1));
            end
            % floe j wrt floe p for mode (0,0)
            T(sum(2*N(1:j-1)+1)*(M+1) + N(j)*(M+1) + 1:...
                sum(2*N(1:j-1)+1)*(M+1) + (N(j)+1)*(M+1),...
                sum(2*N(1:p-1)+1)*(M+1) + N(p)*(M+1) + 1:...
                sum(2*N(1:p-1)+1)*(M+1) + (N(p)+1)*(M+1)) = ...
                sep_matrix(R(p,j), psi(p,j), 0, 0, k);

            % floe p wrt floe j for mode (0,0)
            T(sum(2*N(1:p-1)+1)*(M+1) + N(p)*(M+1) + 1:...
                sum(2*N(1:p-1)+1)*(M+1) + (N(p)+1)*(M+1),...
                sum(2*N(1:j-1)+1)*(M+1) + N(j)*(M+1) + 1:...
                sum(2*N(1:j-1)+1)*(M+1) + (N(j)+1)*(M+1)) = ...
                sep_matrix(R(j,p), psi(j,p), 0, 0, k);
        else % i.e., if N(j)>=N(p)
            for s = -N(p):N(p)
                for n = s+1:N(j)
                    % floe j wrt floe p for mode (n,s)
                    T(sum(2*N(1:j-1)+1)*(M+1) + (n+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (n+N(j)+1)*(M+1),...
                        sum(2*N(1:p-1)+1)*(M+1) + (s+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (s+N(p)+1)*(M+1)) = ...
                        sep_matrix(R(p,j), psi(p,j), n, s, k);

                    % floe j wrt floe p for mode (-n,-s) using symmetry
                    T(sum(2*N(1:j-1)+1)*(M+1) + (-n+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (-n+N(j)+1)*(M+1),...
                        sum(2*N(1:p-1)+1)*(M+1) + (-s+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (-s+N(p)+1)*(M+1)) = ...
                        T(sum(2*N(1:j-1)+1)*(M+1) + (n+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (n+N(j)+1)*(M+1),...
                        sum(2*N(1:p-1)+1)*(M+1) + (s+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (s+N(p)+1)*(M+1))*...
                        exp(1i*(s-n)*(pi-2*psi(p,j)));

                    % floe p wrt floe j for mode (n,s)
                    T(sum(2*N(1:p-1)+1)*(M+1) + (s+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (s+N(p)+1)*(M+1),...
                        sum(2*N(1:j-1)+1)*(M+1) + (n+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (n+N(j)+1)*(M+1)) = ...
                        sep_matrix(R(j,p), psi(j,p), s, n, k);

                    % floe p wrt floe j for mode (-n,-s) using symmetry
                    T(sum(2*N(1:p-1)+1)*(M+1) + (-s+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (-s+N(p)+1)*(M+1),...
                        sum(2*N(1:j-1)+1)*(M+1) + (-n+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (-n+N(j)+1)*(M+1)) = ...
                        T(sum(2*N(1:p-1)+1)*(M+1) + (s+N(p))*(M+1) + 1:...
                        sum(2*N(1:p-1)+1)*(M+1) + (s+N(p)+1)*(M+1),...
                        sum(2*N(1:j-1)+1)*(M+1) + (n+N(j))*(M+1) + 1:...
                        sum(2*N(1:j-1)+1)*(M+1) + (n+N(j)+1)*(M+1))*...
                        exp(1i*(n-s)*(pi - 2*psi(j,p)));
                end
            end
            for s = -N(p):-1
                % floe j wrt floe p for mode (s,s)
                T(sum(2*N(1:j-1)+1)*(M+1) + (s+N(j))*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (s+N(j)+1)*(M+1),...
                    sum(2*N(1:p-1)+1)*(M+1) + (s+N(p))*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (s+N(p)+1)*(M+1)) = ...
                    sep_matrix(R(p,j), psi(p,j), s, s, k);

                % floe j wrt floe p for mode (-s,-s) using symmetry
                T(sum(2*N(1:j-1)+1)*(M+1) + (-s+N(j))*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (-s+N(j)+1)*(M+1),...
                    sum(2*N(1:p-1)+1)*(M+1) + (-s+N(p))*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (-s+N(p)+1)*(M+1)) = ...
                    T(sum(2*N(1:j-1)+1)*(M+1) + (s+N(j))*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (s+N(j)+1)*(M+1),...
                    sum(2*N(1:p-1)+1)*(M+1) + (s+N(p))*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (s+N(p)+1)*(M+1));

                % floe p wrt floe j for mode (s,s)
                T(sum(2*N(1:p-1)+1)*(M+1) + (s+N(p))*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (s+N(p)+1)*(M+1),...
                    sum(2*N(1:j-1)+1)*(M+1) + (s+N(j))*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (s+N(j)+1)*(M+1)) = ...
                    sep_matrix(R(j,p), psi(j,p), s, s, k);

                % floe p wrt floe j for mode (-s,-s) using symmetry
                T(sum(2*N(1:p-1)+1)*(M+1) + (-s+N(p))*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (-s+N(p)+1)*(M+1),...
                    sum(2*N(1:j-1)+1)*(M+1) + (-s+N(j))*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (-s+N(j)+1)*(M+1)) = ...
                    T(sum(2*N(1:p-1)+1)*(M+1) + (s+N(p))*(M+1) + 1:...
                    sum(2*N(1:p-1)+1)*(M+1) + (s+N(p)+1)*(M+1),...
                    sum(2*N(1:j-1)+1)*(M+1) + (s+N(j))*(M+1) + 1:...
                    sum(2*N(1:j-1)+1)*(M+1) + (s+N(j)+1)*(M+1));
            end
            % floe j wrt floe p for mode (0,0)
            T(sum(2*N(1:j-1)+1)*(M+1) + N(j)*(M+1) + 1:...
                sum(2*N(1:j-1)+1)*(M+1) + (N(j)+1)*(M+1),...
                sum(2*N(1:p-1)+1)*(M+1) + N(p)*(M+1) + 1:...
                sum(2*N(1:p-1)+1)*(M+1) + (N(p)+1)*(M+1)) = ...
                sep_matrix(R(p,j), psi(p,j), 0, 0, k);

            % floe p wrt floe j for mode (0,0)
            T(sum(2*N(1:p-1)+1)*(M+1) + N(p)*(M+1) + 1:...
                sum(2*N(1:p-1)+1)*(M+1) + (N(p)+1)*(M+1),...
                sum(2*N(1:j-1)+1)*(M+1) + N(j)*(M+1) + 1:...
                sum(2*N(1:j-1)+1)*(M+1) + (N(j)+1)*(M+1)) = ...
                sep_matrix(R(j,p), psi(j,p), 0, 0, k);
        end
    end
end

%% Routine
function S = sep_matrix(R,psi,n,s,k)

S = diag(besselh(s-n,1,k*R)*exp(1i*(s-n)*psi));
