function energy_budget(A_in, A_R, A_T, chi, Gamma)

% energy_budget(A_in, A_R, A_T, chi)
%
% Computes reflected/transmitted energy to check energy conservation. If
% energy is conserved, the reflected and transmitted coefficients are
% computed.

% Incoming energy
E_In = trapz(chi,abs(A_in).^2);

% Reflected energy
E_R = trapz(chi,abs(A_R).^2);

% Transmitted coefficient
R = sqrt(E_R/E_In);

% Transmitted energy
E_T = trapz(chi,abs(A_T).^2);

% Transmitted coefficient
T = sqrt(E_T/E_In);

% Scattered energy
E_S = E_R + E_T;

if Gamma == 0
    if abs(E_In-E_S) < 1e-5
        disp(':-) Energy is conserved!!!!!!')
        disp(['Reflection coef   = ' num2str(R)])
        disp(['Transmission coef = ' num2str(T)])
    else
        disp('Energy error')
        disp(['Energy in   = ' num2str(E_In)])
        disp(['Energy scat = ' num2str(E_S)])
    end
else
    disp('Energy dissipation experienced')
    disp(['Reflection coef   = ' num2str(R)])
    disp(['Transmission coef = ' num2str(T)])
    disp(['Dissipation rate = ' ...
        num2str(100*(E_In-E_S)/E_In) '% of incoming energy'])
end