function [R_m_ker, T_p_ker, R_p_ker, T_m_ker] = ...
    scat_kernel(pos, Rad, Width, k, M_scat, chi, Ntr, Nev, M, N)

% [R_tr,T_tr,R_ev,T_ev] = scat_kernel(pos, Width, k, M_scat, chi, N, ev)
%
% Computes the reflection and transmission kernel matrices.
%
% In this version of the code, we assume that evanescent vertical modes are
% neglected.

% Init
Np = size(pos,1);

if M>0
    Nev = (length(chi)-Ntr)/2;
    chi_ev = imag([chi(1:Nev) chi(Nev+(Ntr-1)/2+1) chi(Ntr+Nev+1:end)])*1i;
end

%% Cylindrical to plane wave mapping 

Mat_S_m = zeros(Ntr + 2*Nev + M*(2*Nev + 1),Np*(2*N+1)*(M+1));
Mat_S_p = zeros(Ntr + 2*Nev + M*(2*Nev + 1),Np*(2*N+1)*(M+1));
Mat_In_m = zeros(Np*(2*N+1)*(M+1),Ntr + 2*Nev + M*(2*Nev + 1));
Mat_In_p = zeros(Np*(2*N+1)*(M+1),Ntr + 2*Nev + M*(2*Nev + 1));

% Travelling vertical mode
for j = 1:Np
    
    Mat_S_m(:,(j-1)*(2*N+1)+1:j*(2*N+1)) = ...
        exp(-1i*chi.'*(-N:N)).*(exp(1i*k(1)*(pos(j,1)*...
        cos(chi.')-pos(j,2)*sin(chi.')))*...
        (1i.^(-N:N)./besselh(-N:N,1,k(1)*Rad(pos(j,3))))/pi);
    
    Mat_S_p(:,(j-1)*(2*N+1)+1:j*(2*N+1)) = ...
        exp(1i*chi.'*(-N:N)).*(exp(1i*k(1)*((Width-pos(j,1))*...
        cos(chi.')-pos(j,2)*sin(chi.')))*...
        ((-1i).^(-N:N)./besselh(-N:N,1,k(1)*Rad(pos(j,3)))))/pi;
    
    Mat_In_m((j-1)*(2*N+1)+1:j*(2*N+1),:) = ...
        exp(-1i*(-N:N).'*chi).*...
        ((besselj((-N:N).',k(1)*Rad(pos(j,3))).*1i.^(-N:N).')*...
        exp(1i*k(1)*(pos(j,1)*cos(chi)+pos(j,2)*sin(chi))));
    
    Mat_In_p((j-1)*(2*N+1)+1:j*(2*N+1),:) = ...
        exp(1i*(-N:N).'*chi).*...
        ((besselj((-N:N).',k(1)*Rad(pos(j,3))).*(-1i).^(-N:N).')*...
        exp(1i*k(1)*((Width-pos(j,1))*cos(chi)+pos(j,2)*sin(chi))));
end

% Evanescent vertical modes
for m = 2:M+1
    Mat_S_m_m = [];
    Mat_S_p_m = [];
    Mat_In_m_m = [];
    Mat_In_p_m = [];
    for j = 1:Np
        MatSmj = exp(-1i*chi_ev.'*(-N:N)).*(exp(1i*k(m)*(pos(j,1)*...
            cos(chi_ev.')-pos(j,2)*sin(chi_ev.')))*...
            (1i.^(-N:N)./besselh(-N:N,1,k(m)*Rad(pos(j,3))))/pi);
        Mat_S_m_m = [Mat_S_m_m MatSmj];
        
        MatSpj = exp(1i*chi_ev.'*(-N:N)).*...
            (exp(1i*k(m)*((Width-pos(j,1))*cos(chi_ev.') - ...
            pos(j,2)*sin(chi_ev.')))*...
            ((-1i).^(-N:N)./besselh(-N:N,1,k(m)*Rad(pos(j,3)))))/pi;
        Mat_S_p_m = [Mat_S_p_m MatSpj];
        
        MatInmj = exp(-1i*(-N:N).'*chi_ev).*...
            (besselj((-N:N).',k(m)*Rad(pos(j,3))).*1i.^(-N:N).'*...
            exp(1i*k(m)*(pos(j,1)*cos(chi_ev)+pos(j,2)*sin(chi_ev))));
        Mat_In_m_m = [Mat_In_m_m; MatInmj];
        
        MatInpj = exp(1i*(-N:N).'*chi_ev).*...
            ((besselj((-N:N).',k(m)*Rad(pos(j,3))).*(-1i).^(-N:N).')*...
            exp(1i*k(m)*((Width-pos(j,1))*cos(chi_ev) + ...
            pos(j,2)*sin(chi_ev))));
        Mat_In_p_m = [Mat_In_p_m; MatInpj];
    end
    Mat_S_m = blkdiag(Mat_S_m, Mat_S_m_m);
    Mat_S_p = blkdiag(Mat_S_p, Mat_S_p_m);
    Mat_In_m = blkdiag(Mat_In_m, Mat_In_m_m);
    Mat_In_p = blkdiag(Mat_In_p, Mat_In_p_m);
end

%% Row permutatiom matrix
P = zeros(Np*(2*N+1)*(M+1));
Pinv = zeros(Np*(2*N+1)*(M+1));
for j = 1:Np
    for n = -N:N
        for m = 0:M
            P(m*Np*(2*N+1)+(j-1)*(2*N+1)+n+N+1,...
                (j-1)*(2*N+1)*(M+1)+(N+n)*(M+1)+m+1) = 1;
            Pinv((j-1)*(2*N+1)*(M+1)+(N+n)*(M+1)+m+1,...
                m*Np*(2*N+1)+(j-1)*(2*N+1)+n+N+1) = 1;
        end
    end
end
M_scat = P*M_scat*Pinv;

%% Scattering matrices
R_m_ker = Mat_S_m*M_scat*Mat_In_m;
T_p_ker = Mat_S_p*M_scat*Mat_In_m;
R_p_ker = Mat_S_p*M_scat*Mat_In_p;
T_m_ker = Mat_S_m*M_scat*Mat_In_p;

%% Cartesian to polar mapping
% M_scat = M_scat(1:Np*(2*N+1),1:Np*(2*N+1));
% MapC2P_m = M_scat*Mat_In_m;
% MapC2P_p = M_scat*Mat_In_p;