function far_field_circular_MIZ_main

% far_field_circular_MIZ_main
%
% Solves the multiple scattering by a random array of ice floes contained
% in a large disk (circular MIZ) and returns the far field pattern.

%% Parameters
[pos, FSP, R_miz, conc, depth, T, th0, alpha, k0, rho_water, g, E, nu, d, ...
        F, beta, Gamma, gamma, M] = param_gen;
figure('color','w')
subplot(1,2,1)
visu_geom(pos,FSP(:,1))
title(['MIZ radius: ' num2str(round(R_miz)) ' m, # of floes: ' num2str(size(pos,1))])

%% Diffraction transfer matrices (single floes)
D_gen(FSP, depth, T, alpha, nu, beta, d, Gamma, gamma, M);

%% Multiple scattering
[Amps_scat_ext,N] = Multi_scat_solve(pos, FSP, R_miz, depth, T, th0, alpha, ...
    Gamma, M);

%% Compute far-field pattern
theta = linspace(-pi,pi,1001);
ffp = far_field_pattern_floe_array(Amps_scat_ext,N,pos,k0,theta,...
    sqrt(alpha*g),g);

%% Plot far-field pattern
subplot(1,2,2)
polarplot(theta,abs(ffp).^2/(pi*R_miz^2))
title('Scattering kernel')


function visu_geom(pos,rads)

np=size(pos,1);
for i=1:np
    dat = [pos(i,1)*cos(pos(i,2)) - rads(pos(i,3)), ...
        pos(i,1)*sin(pos(i,2)) - rads(pos(i,3)), ...
        2*rads(pos(i,3)), 2*rads(pos(i,3))];
    rectangle('Position',dat,'Curvature',[1 1],'FaceColor',...
        [0.9 0.9 0.9],'EdgeColor','k')%,'LineWidth',1)
end
axis equal
axis off

