function ffp = far_field_pattern_floe_array(Amps_scat_ext,N,pos,k0,theta,om,g)

% ffp = far_field_pattern_floe_array(Amps_scat_ext,N,pos,k0,theta)
%
% Computes the far-field pattern for an array of floating elastic plates

% Number of floes
Np = size(pos,1);

% Polar coordinates of centre of floe j with respect to origin
d   = zeros(Np,1);
psi = zeros(Np,1);
for j = 1:Np
    [d(j),psi(j)] = coordinate_change(...
        [pos(j,1)*cos(pos(j,2)) pos(j,1)*sin(pos(j,2))],[0 0]);
end

% Compute far-field pattern
ffp = zeros(length(theta),1);
for ii = 1:length(theta)
    ffp(ii) = far_field_one_angle(theta(ii),k0,Amps_scat_ext,Np,N,d,psi);
end

% Optical theorem check
LHS = 2*pi/k0*trapz(theta,abs(ffp).^2)
RHS = -4*pi/k0*real(far_field_one_angle(0,k0,Amps_scat_ext,Np,N,d,psi))
LHS/RHS

% Rescaling
ffp = pi*sqrt(pi/(2*k0))*ffp;

function out = far_field_one_angle(th,k0,Amps_scat_ext,Np,N,d,psi)

out = 0;
for jj = 1:Np
    nn = -N(jj):N(jj);
    out = out + exp(1i*k0*d(jj).*cos(psi(jj) - th))*...
        exp(1i*nn*th)*diag((-1i).^nn)*...
        Amps_scat_ext(sum(2*N(1:jj-1)+1)+1:sum(2*N(1:jj)+1));
end
out = out/pi;