function [pos, FSP, R_miz, conc, depth, T, th0, alpha, k0, ...
    rho_water, g, E, nu, d, F, beta, Gamma, gamma, M] = param_gen

% param_gen
%
% Generates an array of parameters

%% Geometry

% FSD
[pos, FSP, R_miz, conc] = FSD_gen;



% Water depth (m)
depth = 200;

%% Forcing

% Wave period
T = 10;

% Incident angle (wrt positive x direction)
th0 = 0;

%% Physics
% Fluid density (kg/m^3)
rho_water = 1025;

% Accelaration due to gravity (m/s^2)
g = 9.81;

% Young's modulus (Pa)
E = 6e9;

% Density ice (kg/m^3)
rho = 922.5;

% Poisson's ratio
nu = 0.3;

% Draught
d = FSP(:,2).*rho./rho_water;

% Flexural Rigidity
F = E.*FSP(:,2).^3/(12*(1-nu^2));

% Stiffness
beta = F./(rho_water*g);

% Viscosity (Pa.s/m)
Gamma = 0;

% viscous parameter
gamma = Gamma/(rho_water*sqrt(g));

% Infinite depth wavenumber
alpha = (2*pi/T).^2/g;

% Finite depth deep water
k0 = dispersion_free_surface(alpha,0,depth);

%% Numerical method
% Number of vertical modes
M = 0;

% % Number of angular modes
% N = 5;


% %% Save parameters
% save param_run pos FSP R_miz depth T alpha ...
%     rho_water g E nu d F beta Gamma gamma M N